/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*     parsedrop.h:  header file for Pars/proc_drop()                         */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef PARSEDROP_H
#define PARSEDROP_H

/**
 * @addtogroup GDrop
 * @{
 */

/*            built by mak_tloc_list():                                       */

struct tllist_node {                    /*Tllist_node:                        */
    int no;                             /*tloc no. (in numerical order)       */
    int n_us;                           /*no uses of this tloc (info only)    */
    int lnkd;                           /*mapped,  unlinked, or varying pos?  */
    struct tllist_node * p_next;
                  };
typedef struct tllist_node TLLIST;
typedef TLLIST * TLLIST_PTR;
static TLLIST_PTR first_tllist_node = NULL;

/*            built by add_prntmap(), add_only_mark_locus(), add_mark_locus() */

struct prntmap_node {                  /*Prntmap_node:                        */
    int chrom_no;                      /*chromosome no.                       */
    int no_mark_sel;                   /*no. markers simulated/selected       */
    int no_mark_map;                   /*no. markers in map                   */
    int no_tloc_sel;                   /*no. tlocs simulated/selected         */
    struct prntmap_locus * p_locus;
    struct prntmap_node * p_next;
                    };
typedef struct prntmap_node PRNTMAP;
typedef PRNTMAP * PRNTMAP_PTR;
static PRNTMAP_PTR first_prntmap_node[3] = {NULL, NULL, NULL};

/*            step 1 built by add_only_mark_locus(), add_mark_locus(),        */
/*                            add_tloc_locus()                                */
/*            step 2          sel_mark_locus(), sel_tloc_locus()              */
/*            step 3          shrink_prntmap(), fix_locus_dist()              */

struct prntmap_locus {                 /*Prntmap_locus:                       */
                                       /*  step 1:                            */
    int locus_typ;                     /*is this locus marker or tloc?        */
    int no;                            /*marker or tloc no.                   */
    int map_mark;                      /*marker for dist or recomb            */
    double dist;                       /*dist to adjacent marker              */
    double m1_dist;                    /*dist to marker 1                     */
                                       /*  step 2:                            */
    int selectd;                       /*relative selected marker no. or 0    */
    int sel_map_mark;                  /*marker no. for sel_dist              */
    double sel_dist;                   /*dist to selected marker              */
                                       /*  step 3:                            */
    double loc_dist;                   /*dist to prev locus for printing map  */
    struct prntmap_locus * p_prev;
    struct prntmap_locus * p_next;
                     };
typedef struct prntmap_locus PRDIST;
typedef PRDIST * PRDIST_PTR;


static int   loc_spac = 7;             /*no. spaces formatted per locus       */
static int   max_line_loc = 11;        /*up to 11 loci per map segment        */
static char  trait_line[82];           /*with 5 lines formatted for ea        */
static char  trait_tic_marks[82];      /*chrom segment                        */
static char  dist_line[3][82] = {"F  ", "M  ", "   "};
static char  mark_tic_marks[82];
static char  mark_line[84];


/*                                    function prototypes for proc_dropf.c    */
static PRDIST_PTR add_locus_node (void);
static PRDIST_PTR add_mark_locus (PRNTMAP_PTR, DIST_PTR);
static PRDIST_PTR add_only_mark_locus (PRNTMAP_PTR);
static PRDIST_PTR add_tloc_locus (PRNTMAP_PTR, DIST_PTR);
static PRNTMAP_PTR add_prntmap (SIM_PTR, int);
static PRNTMAP_PTR mak_markprmap (SIM_PTR, int);
static int chk_markmindist (PRNTMAP_PTR);
static int chk_tlocprmap (PRNTMAP_PTR);
static int find_inh_tloc_map (void);
static int find_inher_tloc (void);
static int find_tloc_map (void);
static int find_tr_tloc_map (void);
static int find_trait_tloc (void);
static int test_loc_mindist (PRDIST_PTR, double);
static int test_mark_freq (SIM_PTR, int);
static int test_mark_map (SIM_PTR, int *);
static int test_sim (void);
static int test_tloc_dist (PRNTMAP_PTR, PRDIST_PTR);
static int test_tloc_freq (int);
static int test_tloc_map (SIM_PTR, int *);
static int test_tr_tl_genotyp (int, int);
static int test_trait_vars (int);
static int updt_sim_lnkd_tloc (void);
static int updt_sim_unln_tloc (void);
static void alloc_recomb (void);
static void alloc_tloc_out (void);
static void alloc_traits (int);
static void find_nxt_tl_map (int);
static void fix_locus_dist (PRNTMAP_PTR, PRDIST_PTR);
static void fix_prmap_dist (PRNTMAP_PTR);
static void fmt_tloc_freqs (char *, int, int);
static void fmt_trait_vari (char *, int);
static void fmt_trtl_geno_means (int, int);
static void for_chrom (int);
static void for_chrom_mark (int, int);
/*static void for_chrom_tloc (int, int);                                      */
static void for_trait_at_tloc (int, int);
static void mak_freq (PRNTMAP_PTR, PRDIST_PTR, int, int);
static void mak_no_recomb (int, int, int);
static void mak_recomb (PRDIST_PTR, int, int);
static void mak_tlocprmap (SIM_PTR, PRNTMAP_PTR, int);
static void mak_tloc_list (int, int);
static void mak_tloc_out (int, int, int);
static void mak_trait (int, int, int, int);
static void on_chrom (int);
static void prnt_chrom_head (SIM_PTR);
static void prnt_inher_head (void);
static void prnt_inh_tl_list (void);
static void prnt_map (int);
static void prnt_mark_freqs (SIM_PTR);
static void prnt_tloc_freqs (PRNTMAP_PTR);
static void prnt_tr_list (void);
static void prnt_tr_tl_genotyp (void);
static void prnt_tr_tl_list (void);
static void prnt_trait_head (void);
static void prnt_trait_vari (void);
static void prnt_unltloc_freqs (void);
static void prnt_unltloc_head (void);
static void prnt_unltloc_list(void);
static void prntmap_dist (int, int, double);
static void prntmap_line (int, int);
static void prntmap_mrk (int, int);
static void prntmap_mrk0 (int);
static void prntmap_tl (int, int);
static void prntmap_tl0 (int);
static void prntmap_tl0b (void);
static void prntvmap_tl (int, int);
static void prntvmap_tl0 (int);
static void sel_mark_locus (SIM_PTR, PRDIST_PTR, int *);
static void sel_tloc_locus (PRDIST_PTR);
static void set_alle_frq_deflts (void);
static void trim_blanks (char *);

/*                                    function prototypes for proc_nxt.c      */
static DIST_PTR nxt_dist (int *, MAP_PTR);
static DIST_PTR nxt_sim_mark_dist (int *, int, SIM_PTR);
static DIST_PTR nxt_sim_tl_dist (int *, int, SIM_PTR);
static INHERTLOC_PTR nxt_inh_tl (int *);
static MAP_PTR nxt_mark_map (int *, int);
static MAP_PTR nxt_tl_map (int *, int);
static PRDIST_PTR nxt_prntmap_locus (int *, PRNTMAP_PTR);
static PRNTMAP_PTR nxt_prntmap (int *, int);
static SIM_PTR nxt_sim_chrom (int *);
static TRAITEFF_PTR nxt_tr_eff (int *);
static TRAITTLOC_PTR nxt_tr_tl (int *);
static UDIST_PTR nxt_unln_dist (int *);
static int get_tl_sim_loc_no (int);
static int nxt_inh_tl_no (int *, int);
static int nxt_sim_mapd_tl_no (int *, SIM_PTR);
static int nxt_sim_mark_no (int *, SIM_PTR);
static int nxt_sim_tl_no (int *);
static int nxt_sim_tr_no (int *);
static int nxt_sim_unln_tl_no (int *);
/*static int nxt_tr_eff_no (int *, int);                                      */
static int nxt_tr_tl_no (int *, int);

/** @} */

#endif  /*PARSEDROP_H*/
