/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       stuff.h:  header file for Stuff/                              */
/*                 this header file includes:                          */
/*                    data structures for Stuff/ functions             */
/*                    function prototyes for Stuff/ functions          */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef STUFF_H
#define STUFF_H

#include "incs.h"
#include "f2c.h"

/**
 * @defgroup stuff libstuff
 * @ingroup Libraries
 * @{
 */

#define DATASINK "/dev/null"

#define WHERE    __FILE__, __LINE__

#ifndef BB
#define BB(b,i,j) ((b)[i*(i+1)/2+j])
#endif

GLOBAL char name_format[8];             /* Name format used by read_name.     */

GLOBAL int nWarns;                      /*total no. of warnings issued  */

                                        /*table of program codes vs.    */
                                        /*functions to be called        */
                                        /*(program codes defined in     */
                                        /*Filtab[] in globs.h           */
struct _cleanup {                       /*table contents defined in     */
       char * pcods;                    /*main program headers)         */
       void (* func) (void);
                };
GLOBAL struct _cleanup * PCleanUp;
                                        /*function prototypes for       */
                                        /*Stuff/ library:               */
int qfunc_doub  (const void *, const void *);
int qfunc_int   (const void *, const void *);
int qfunc_doubdec (const void *, const void *);
int comparNam     (const void *, const void *);

void alloc_chk       (char *, int, void *);
void give_warn       (int, ...);
void give_warn_where (char *, int, int, ...);
void give_err        (int, ...);
void give_err_where  (char *, int, int, ...);
void give_up         (int, ...);
void give_up_where   (char *, int, int, ...);

void initMorgProg (char **);
void print_head (char *),            print_tail (void);
void printhost  (void);
void print_val  (double),            print_val5 (double);

void get_colfmt  (int, int, int, int, int *, int *);
int  get_colfmt2 (int, int, int, int);
int  fmt_marknam (char *, int);
int  fmt_tlocnam (char *, int);
int  fmt_traitnam(char *, int);
void prnt_locnam (int, int *);
void fmt_locnamlin (char *, int *, int, int *);

int  intWid     (int, int),          decWid     (double, double, int);
int  fldWid     (double, double, int);
char * dub2nlz  (double, int, int);

double dist2rf  (double),            rf2dist    (double);
double distK2rf (double),            rf2distK   (double);
double gendist  (double),            recomb     (double);

void prntStatusPP  (void),    prntStatusCP  (void),
     prntStatusCPF (void),    prntStatusSMC (void),
     prntStatusIMC (char *),  prntStatusBMC (void),
     prntStatusFMC (void);

FILE * open_fil (char *);
FILE * open_ped (void);
FILE * open_chk (char *, int, char *, char *);
FILE * open_chkq (char *, int, char *, char *);
FILTAB_PTR get_filent (char *);
FILTAB_PTR get_filtyp (char);
void   free_fnam (FILTAB_PTR);
void   free_fnams (void);
void   free_drop (void);
void   free_auto (void);
void   free_marek (void);
void   free_dub (void);
void   clean_morg (void);

void   free_rescue_filenames (char **);

/* Allocate 1D, 2D or 3D arrays and initialize array elements to 0 or 0.0.    */
int      *alloc_int1d (int);
int     **alloc_int2d (int, int);
int    ***alloc_int3d (int, int, int);
double   *alloc_dbl1d (int);
double  **alloc_dbl2d (int, int);
double ***alloc_dbl3d (int, int, int);
char    *alloc_char1d (int);
char   **alloc_char2d (int, int);
char  ***alloc_char3d (int, int, int);

/* Free array.  Assign null to the pointer which was pointing to the array.   */
void          free_1d (void **);
void          free_2d (void ***);
void          free_3d (void ****);

/* Liping's routines to allocate one dimensional vectors.                     */
void al_int_vector (int **, int);
void al_double_vector (double **, int);

/**
 * Return the map position (in cM) of current (varying) trait.
 *
 * @param pos Current position index, 0 .. NTPos - 2.
 * @param sex Indicate which map to use, 0 for female and 1 for male map.
 *
 * @return current trait position in centi-Morgan (female or male map).
 */
double map_position (int pos, int sex);

/**
 * Given matrices of likelihood ratios, compute and print lod scores.
 * Three methods are used to compute lod scores from the 'left' and 'right'
 * ratios:
 *
 * - Simple averaging from the left.
 * - Simple averaging from the right.
 * - Solve the eigensystem defined by the sparse matrix, return the
 *   eigenvector corresponding to the largers real eigenvalue.  Also print
 *   out the second largest real eigenvalue.
 *
 * Output is sent to stdout.
 *
 * @param lr A array of @ref n_compnts sparse matrices which store all the
 *           left and right likelihood ratios (@ref NTPos x @ref NTPos
 *           matrices).
 */
void print_lods (double *** lr);

/** @} */

#endif  /*STUFF_H*/
