/* Program to do a basic analysis of IBD inference with no LD.                */
/* Up to 4 genomes, or genotypic pairs.                                       */

/* Also includes options to simulate the ibd status of chromosomes.           */
/* Initially these will just be segregants from first cousins.                */
/* We will make the chromosomes 2 Morgans, so most have an IBD segment.       */

#define DEFINE_DATA
#define PNAM "ibd_haplo"

#include "parsie.h"
#include "rans.h"
#include "ibd_haplo.h"

#include "haplo_process.c"
#include "hap_fwd_bk_compute.c"

int main (int argc, char *argv[])
{
   /* When the data are partially phased, we can use Morgan's extra input     */
   /* file for reading in the partial phasing data.  This will be the only    */
   /* situation where the iXtra FILE variable will be needed.                 */

   FILE *oXtra, *iXtra;

   double *snppos;                      /* Selected marker positions.         */
   double *allefreq;                    /* Allele frequencies at each marker. */

   double **qq_ibd_final;
   double **qq_ibd;
   double **pprob;

   double ffibd[15];                    /* Overall inbr,  Marginal ibd prob.  */

   char  temp_buff[8];                  /* A temporary buffer to format the   */
   /*                                      highest selected marker number to. */

   /* Arrays of phasing data.                                                 */
   int **phasdatindiv;                  /* First is for individuals.          */
   int **phasdat;                       /* Second is for pairs of individuals. */

   int  *phasptr = NULL;

   int   ns, nph, nstmp;                /* Number of states, and phenotypes.  */
   int   nhap;                          /* Number of haplos, and start and    */
   /*                                      end of SNP block.                  */

   int **haplodat;                      /* Haplotypic data.                   */
   int  *hapdat;

   int   indiv;                         /* Subscript for current individual.  */

   int   tmp1, tmp2;                    /* The two tmp variables are holders  */
   /*                                      for swapping alleles.              */

   int   nc;                            /* Character length of longest name.  */
   int   nm;                            /* Character length of highest        */
   /*                                      numbered selected marker.          */

   int   half_hss;                      /* HapSetSize / 2.                    */
   int   hs_ndx;                        /* Current offset into ObsNam for the */
   /*                                      current haplotype set.             */

   int   i, j, k;

/******************************************************************************/
/*                                                                            */
/* The executable portion of the main program starts here.                    */
/*                                                                            */
/******************************************************************************/

   initMorgProg (argv);

   /* Parse input parameter statements.                                       */
   if (!proc_pars (argc, argv, PNAM))
      exit (EXIT_FAILURE);

   prntStatusPP ();

   /* Check statements for completeness and consistency.                      */
   /* Set up global parameters and data structures.                           */
   if (!proc_auto ())
      exit (EXIT_FAILURE);

   prntStatusCP ();

   if (PhaseofData == UNPHASED)         /* Perform a genotypic analysis.      */
   {
      ns = 9;
      nph = 9;
   }
   else                                 /* Perform a haplotypic analysis.     */
   {
      ns = 15;
      nph = 16;
   }

   if (HapSetSize == 2)                 /* Perform a pairwise analysis.       */
   {
      ns = 2;
      nph = 4;
   }

/************************* TEMP TEMP TEMP TEMP FIX ME *************************/
   skipmark = 1;

   nhap = NumberHapSets * HapSetSize;

   /* Allocate storage for the numerous arrays we will need.                  */

   snppos = malloc (NMarks * sizeof (double));
   haplodat = alloc_int2d (nhap, NMarks);

   hapdat = malloc (NMarks * sizeof (int)); /* All cases: analysis data.      */
   allefreq = malloc (NMarks * sizeof (double));

   transmat = alloc_dbl2d (ns, ns);
   qq_ibd = alloc_dbl2d (NMarks, ns);
   pprob = alloc_dbl2d (NMarks, ns);

   /* Copy the genotypic data from the ObsMark[][][] array.                   */

   for (i = 0; i < nhap; i = i + 2)
   {
      indiv = i / 2;
      for (j = 0; j < NMarks; ++j)
      {
         haplodat[i][j] = ObsMark[indiv][j][0];
         haplodat[i + 1][j] = ObsMark[indiv][j][1];
      }
   }

   /* Initialize the integer variable, nc, to the number of characters in the */
   /* longest name of an observed individual in the dataset.                  */

   nc = 0;
   if (NObs > 0)
   {
      for (i = 0; i < NObs; ++i)
      {
         j = strlen (ObsNam[i]);
         nc = j > nc ? j : nc;
      }
   }

   /* To make the data written to the extra output file line up nicely, find  */
   /* out how many characters are needed to write the marker number of the    */
   /* highest numbered selected marker.                                       */

   sprintf (temp_buff, "%d", MarSel[NMarks - 1]);
   nm = strlen (temp_buff);

   if (PhaseofData == PARTIALPHASE)
   {
      /* One line of phasing data per haplotypic pair.                        */
      phasdatindiv = alloc_int2d (nhap / 2, NMarks);

      /* Could ditch this matrix and just compute the vector for each compute run. */
      phasdat = alloc_int2d (NumberHapSets, NMarks);

      /* Open the extra input file to read in the partial phasing data.       */

      if (!(iXtra = open_fil (XTRA)))
         give_up (ERR_qit);

      printf (" Reading phasing status data:");

      for (i = 0; i < nhap / 2; ++i)
      {
         fscanf (iXtra, "%*d %*d");
         for (j = 0; j < NMarks; ++j)
         {
            fscanf (iXtra, "%d", &(phasdatindiv[i][j]));
            if (phasdatindiv[i][j] == DISCORDANT)
            {
               tmp1 = haplodat[2 * i][j];
               tmp2 = haplodat[2 * i + 1][j];
               haplodat[2 * i][j] = tmp2;
               haplodat[2 * i + 1][j] = tmp1;
               phasdatindiv[i][j] = 1;
            }
         }
      }

      /* Close the extra input file since we are done reading in the partial  */
      /* phasing data.                                                        */
      fclose (iXtra);

      for (i = 0; i < NumberHapSets; ++i)
      {
         for (j = 0; j < NMarks; ++j)
         {
            /* It's only phased for the HMM model if BOTH are phased.         */
            phasdat[i][j] = phasdatindiv[2 * i][j] * phasdatindiv[2 * i + 1][j];
         }
      }

      for (i = 0; i < nhap / 2; ++i)
      {
         printf ("\n Individual name %*s:", nc, ObsNam[i]);
         for (j = 0; j < 10; ++j)
            printf (" %d", phasdatindiv[i][j]);
      }

      for (i = 0; i < NumberHapSets; ++i)
      {
         printf ("\n Pair (4-haplotypes) %*s %*s:", nc, ObsNam[2 * i], nc,
                 ObsNam[2 * i + 1]);
         for (j = 0; j < 10; ++j)
            printf (" %d", phasdat[i][j]);
      }

      printf ("\n\n\n");
      fflush (stdout);
   }

   /* Compute the positions of all the markers involved in the analysis.      */

   snppos[0] = FirstMarkPos;
   for (j = 1; j < NMarks; ++j)
      snppos[j] = snppos[j - 1] + Mdist[j - 1];

   /* Copy the allele frequencies from the global variable, Qq, to the local  */
   /* variable, allefreq[].                                                   */

   for (j = 0; j < NMarks; ++j)
      allefreq[j] = Qq[j + 1][1];

   /* Open the extra output file.                                             */

   if (!(oXtra = open_fil (OXTR)))
      give_up (ERR_qit);

   /* Make the data vector of states from the haplotypic data.                */
   /* First, haplodat has switched from 0,1 to 1,2 (with 0 missing).          */
   /* Second, we can now do several groups of 4 haplotypes: for 1 set kk = 0. */

   hh = 1 - PopKinship;
   gg = PopKinship;

   set_marg_ibd (ns, ffibd);
   printf (" Equilib state probs at kinship %f\n", PopKinship);
   for (i = 0; i < ns; ++i)
      printf (" %5.3f", ffibd[i]);
   printf ("\n");
   fflush (stdout);

   for (k = 0; k < NumberHapSets; ++k)  /* k'th set of haplotypes.            */
   {
      printf ("\n Data vector for %d markers in set %d to be set", NMarks,
              k + 1);
      fflush (stdout);
      make_data (hapdat, haplodat, HapSetSize, k, NMarks, 1, 0);

      printf ("\n Data vector for %d markers in set %d is set", NMarks, k + 1);
      fflush (stdout);

      set_transmat (ns, transmat, ffibd, 1.0); /* Check transmat.             */
      for (i = 0; i < ns; ++i)
      {
         printf ("\n");
         for (j = 0; j < ns; ++j)
            printf (" %7.5f", transmat[i][j]);
      }

      printf ("\n Markers %d, phen %d , states %d", NMarks, nph, ns);

      /* If using the matrix of phased data, set the relevant pointer.        */
      if (PhaseofData == PARTIALPHASE)
         phasptr = phasdat[k];

      hap_fwd_compute (hapdat, NMarks, nph, ns, transmat, allefreq,
                       ffibd, Mdist, pprob, PhaseofData, phasptr);
      hap_bck_compute (hapdat, NMarks, nph, ns, qq_ibd, transmat,
                       allefreq, ffibd, Mdist, pprob, PhaseofData, phasptr);

      /* The output written to both the standard output file and qibd file    */
      /* are prefixed by printing the number of markers and the names of the  */
      /* individuals included in each analysis.                               */

      fprintf (stdout, "\n\n Markers = %d\n", NMarks);
      fprintf (oXtra, "Markers = %d\n", NMarks);

      fprintf (stdout, " Names:");
      fprintf (oXtra, " Individual names:");

      /* Each individual has 2 haplotypes, so for printing out names we will  */
      /* need an integer set to half of HapSetSize.                           */

      half_hss = HapSetSize / 2;

      for (hs_ndx = 0; hs_ndx < half_hss; ++hs_ndx)
      {
         fprintf (stdout, " %s", ObsNam[half_hss * k + hs_ndx]);
         fprintf (oXtra, " %s", ObsNam[half_hss * k + hs_ndx]);
      }

      fprintf (stdout, "\n");
      fprintf (oXtra, "\n");

      fflush (stdout);
      fflush (oXtra);

      /* If partially phased data, collapse 15 states down to 9.              */
      if (PhaseofData == PARTIALPHASE)
      {
         qq_ibd_final = alloc_dbl2d (NMarks, 9);
         for (i = 0; i < NMarks; ++i)
         {
            qq_ibd_final[i][0] = qq_ibd[i][0];
            qq_ibd_final[i][1] = qq_ibd[i][1];
            qq_ibd_final[i][2] = qq_ibd[i][2] + qq_ibd[i][3];
            qq_ibd_final[i][3] = qq_ibd[i][4];
            qq_ibd_final[i][4] = qq_ibd[i][5] + qq_ibd[i][6];
            qq_ibd_final[i][5] = qq_ibd[i][7];
            qq_ibd_final[i][6] = qq_ibd[i][8] + qq_ibd[i][9];
            qq_ibd_final[i][7] =
               qq_ibd[i][10] + qq_ibd[i][11] + qq_ibd[i][12] + qq_ibd[i][13];
            qq_ibd_final[i][8] = qq_ibd[i][14];
         }
         nstmp = 9;
      }
      else
      {
         qq_ibd_final = qq_ibd;
         nstmp = ns;
      }

      for (i = 0; i < NMarks; ++i)
      {
         fprintf (oXtra, " %*d  %10.6f ", nm, MarSel[i], snppos[i]);
         for (j = 0; j < nstmp; ++j)
            fprintf (oXtra, " %7.4f", qq_ibd_final[i][j]);
         fprintf (oXtra, "\n");
      }
   }

   /* Close the extra output file.                                            */

   fflush (oXtra);
   fclose (oXtra);

   /* We are done, so all of the allocated storage can now be freed.          */

   free (snppos);
   free (haplodat);

   free (hapdat);
   free (allefreq);

   free (transmat);
   free (qq_ibd);
   free (pprob);

   if (PhaseofData == PARTIALPHASE)
   {
      free (phasdatindiv);
      free (phasdat);
      free (qq_ibd_final);
   }

   free_auto ();
   proc_free ();

   exit (EXIT_SUCCESS);
}
