/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

void write_rb (FILE * fptr_rb, double **rao_realiz)
{
   int   i;

   for (i = 0; i < NTPos; i++)
      fprintf (fptr_rb, " %e ", (*rao_realiz)[i]);
   fprintf (fptr_rb, " %e\n ", (*rao_realiz)[NTPos]);

   return;
}

void set_previous_realiz (int intvl, int tposit, double fem_rec_lft,
                          double mal_rec_lft, double prb_Yt_condtl,
                          int **xx0, int **xx1, int nummrks, int nped)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*  Written: 4th February, 2001.                                              */
/*  Purpose: Initializing data structure for previous state of markov chain.  */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
{
   int   ind;
   int   locus;

/* Set interval in which trait locus lies.                                    */
   previous_realiz.intvl = intvl;
/* Set position of trait locus within marker interval.                        */
   previous_realiz.tposit = tposit;
/* Female and male recombination rates of trait to left marker.               */
   previous_realiz.rec_lft[0] = fem_rec_lft;
   previous_realiz.rec_lft[1] = mal_rec_lft;
/* Set P(Y_T | S_intvl, S_{intvl+1}, \lamba_T).                               */
   previous_realiz.prb_Yt_condtl = prb_Yt_condtl;
   for (locus = 0; locus <= nummrks; locus++)
      for (ind = 0; ind < nped; ind++)
      {
         previous_realiz.xx0[locus][ind] = xx0[locus][ind];
         previous_realiz.xx1[locus][ind] = xx1[locus][ind];
      }

   return;
}

void calc_raoI (int nummrks, int rao_freq, double *rao_realiz)
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/*  Version 1.1: Outputing rao_realiz.                                        */
/*  This is my first attempt at coming up with a Rao-Blackwellize             */
/*  estimator of P(\lamba_T | Y)/P(\lamba_0 | Y).                             */
/*  The estimator is

        pi(\lamba_T | Y)
        --------------------   =
        pi(\lamba_0 | Y)

        E_P [  P(Y_T | S_M, lamba_T)P(lamba_T) / sum_{lamba_T)
        ------------------------------------------------------ X
        E_P [  P(Y_T | S_M, lamba_0)P(lamba_0) / sum_{lamba_T)

        P(Y_T | S_M, lamba_T)P(lamba_T)  |  Y]
        ---------------------------------------------
        P(Y_T | S_M, lamba_T)P(lamba_T)  |  Y]                                */
{

   /* Variable declaration.                                                   */
   int   sx;
   int   ii, jj;
   int  *locusordr_cpy;                 /* Copy of the global chromo where
                                           chromo[0][k] = locusordr_cpy[k].   */

   double **locusrec_cpy;               /* Copy of the global xecomo where
                                           xecomo[sx][0][k] =
                                           locusrec_cpy[k].                   */
   double nc;                           /* Normalization constant.            */
   double recenter;

   locusrec_cpy = calloc (sizeof (double), 2);
   alloc_chk (WHERE, locusrec_cpy);
   for (sx = FEM; sx <= MAL; sx++)
   {
      locusrec_cpy[sx] = calloc (nummrks + 1, sizeof (double));
      alloc_chk (WHERE, locusrec_cpy[sx]);
   }

   locusordr_cpy = calloc (sizeof (int), nummrks + 1);
   alloc_chk (WHERE, locusordr_cpy);

   /* Make a copy of the globals so that they can be restored upon exit.      */
   for (sx = FEM; sx <= MAL; sx++)
      for (jj = 0; jj <= nummrks; jj++)
         locusrec_cpy[sx][jj] = xecomo[sx][0][jj];

   for (jj = 0; jj <= nummrks; jj++)
      locusordr_cpy[jj] = chromo[0][jj];

   /* Calculate log P(Y_T | S_M, \lamba_T) across ALL test positions
      (including unlinked).                                                   */
   for (ii = 0; ii < NTPos1; ii++)      /* Remember that NTPos1 is 1 + the
                                           number of test positions.          */
   {
      for (sx = FEM; sx <= MAL; sx++)
         set_chromosomes (nummrks, Marek[sx], 0, &IntVal[ii],
                          &RecVal[sx][ii], sx);
      set_positions (nummrks);          /* Assign loci for L sampler.         */
      set_transmission (loci, 0, nummrks + 1);
      rao_realiz[ii] = locus_mjrpeel (1.0, 1.0, 0, loci);

      /* Peel only.                                                           */
      /* Checks for zero probability in peeling operation.                    */
      if (rao_realiz[ii] == -DBL_MAX)
         give_up_where (WHERE, ERS_zpp);
   }

   /* To minimize over/under flow errors, recenter values.                    */
   recenter = rao_realiz[0] + log (PseudoPrior[0]);

   for (ii = 0; ii < NTPos1; ii++)
      rao_realiz[ii] = exp (rao_realiz[ii] + log (PseudoPrior[ii]) - recenter);

   /* Calculate \sum_{\lamba_T} [P(Y_T | S_M, \lamba_T) p(\lamba_T)] for      */
   /* R.B. estimator.                                                         */
   nc = 0.0;
   for (ii = 0; ii < NTPos1; ii++)
      nc += rao_realiz[ii];

   for (ii = 0; ii < NTPos1; ii++)
      rao_realiz[ii] = rao_realiz[ii] / nc;

   /* Incremental expectation.                                                */
   for (ii = 0; ii < NTPos1; ii++)
      raoI_est[ii] = (rao_freq - 1) * raoI_est[ii] / rao_freq
         + rao_realiz[ii] / rao_freq;

   /* Restore globals and loci.                                               */
   for (sx = FEM; sx <= MAL; sx++)
      for (jj = 0; jj <= nummrks; jj++)
         xecomo[sx][0][jj] = locusrec_cpy[sx][jj];

   for (jj = 0; jj <= nummrks; jj++)
      chromo[0][jj] = locusordr_cpy[jj];

   set_positions (nummrks);             /* Restore loci.                      */

   /* Free up memory.                                                         */
   for (sx = FEM; sx <= MAL; sx++)
      free ((void **) locusrec_cpy[sx]);
   free ((void *) locusrec_cpy);
   free ((void *) locusordr_cpy);

   return;
}
