/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "markers.h"

/*  these malloc and free functions malloc arrays with sizes based 
 *  on the number of components and/or loci
 *  the allocation functions are:
 *
 *  ali_nc             int [nc]
 *  ali_nl             int [nl]
 *  ali_n_nl           int [n][nl]
 *  ali_nc_n           int [nc][n]
 *  ali_nl_n           int [nl][n]
 *  ali_nc_nl          int [nc][nl]
 *  ali_nc_nl_n        int [nc][nl][n]
 *  ald_nl             double [nl]                         
 *  ald_nc_n           double [nc][n]
 *  ald_nl_n           double [nl][n]
 *  ald_nc_nl          double [nc][nl]                         
 *  ald_nc_nl_ns       double [nc][nl][ns] 
 *  ald_nc_nl_n        double [nc][nl][n] 
 *
 *  the free functions are:
 *
 *  fre_n              [n]          (int or double)
 *  free_n_n           [n][nl]
 *  fre_nc_n           [nc][n] 
 *  fre_nl_n           [nl][n]
 *  fre_nc_nl_n        [nc][nl][n]
 */

void ali_nc (int **h)
{
   *h = malloc (n_compnts * sizeof (int));
   alloc_chk (WHERE, *h);

   return;
}
void ali_nl (int **h)
{
   *h = malloc (NLoci * sizeof (int));
   alloc_chk (WHERE, *h);

   return;
}
void ali_n_nl (int ***h, int n)
{
   int   i;

   *h = malloc (n * sizeof (int *));
   alloc_chk (WHERE, *h);
   for (i = 0; i < n; ++i)
   {
      (*h)[i] = malloc (NLoci * sizeof (int));
      alloc_chk (WHERE, (*h)[i]);
   }
   return;
}
void ali_nc_n (int ***h, int n)
{
   int   i;

   *h = malloc (n_compnts * sizeof (int *));
   alloc_chk (WHERE, *h);
   for (i = 0; i < n_compnts; ++i)
   {
      (*h)[i] = malloc (n * sizeof (int));
      alloc_chk (WHERE, (*h)[i]);
   }
   return;
}
void ali_nl_n (int ***h, int n)
{
   int   i;

   *h = malloc (NLoci * sizeof (int *));
   alloc_chk (WHERE, *h);
   for (i = 0; i < NLoci; ++i)
   {
      (*h)[i] = malloc (n * sizeof (int));
      alloc_chk (WHERE, (*h)[i]);
   }
   return;
}
void ali_nc_nl (int ***h)
{
   int   i;

   *h = malloc (n_compnts * sizeof (int *));
   alloc_chk (WHERE, *h);
   for (i = 0; i < n_compnts; ++i)
   {
      (*h)[i] = malloc (NLoci * sizeof (int));
      alloc_chk (WHERE, (*h)[i]);
   }
   return;
}
void ali_nc_nl_n (int ****h, int n)
{
   int   i, j;

   *h = malloc (n_compnts * sizeof (int **));
   alloc_chk (WHERE, *h);
   for (i = 0; i < n_compnts; ++i)
   {
      (*h)[i] = malloc (NLoci * sizeof (int *));
      alloc_chk (WHERE, (*h)[i]);
      for (j = 0; j < NLoci; ++j)
      {
         (*h)[i][j] = malloc (n * sizeof (int));
         alloc_chk (WHERE, (*h)[i][j]);
      }
   }
   return;
}
void ald_nl (double **d)
{
   *d = malloc (NLoci * sizeof (double));
   alloc_chk (WHERE, *d);

   return;
}
void ald_nc_n (double ***d, int n)
{
   int   i;

   *d = malloc (n_compnts * sizeof (double *));
   alloc_chk (WHERE, *d);
   for (i = 0; i < n_compnts; ++i)
   {
      (*d)[i] = malloc (n * sizeof (double));
      alloc_chk (WHERE, (*d)[i]);
   }
   return;
}
void ald_nl_n (double ***d, int n)
{
   int   i;

   *d = malloc (NLoci * sizeof (double *));
   alloc_chk (WHERE, *d);
   for (i = 0; i < NLoci; ++i)
   {
      (*d)[i] = malloc (n * sizeof (double));
      alloc_chk (WHERE, (*d)[i]);
   }
   return;
}
void ald_nc_nl (double ***d)
{
   int   i;

   *d = malloc (n_compnts * sizeof (double *));
   alloc_chk (WHERE, *d);
   for (i = 0; i < n_compnts; ++i)
   {
      (*d)[i] = malloc (NLoci * sizeof (double));
      alloc_chk (WHERE, (*d)[i]);
   }
   return;
}
void ald_nc_nl_ns (double ****d)
{
   /* mallocs double array of size [n components][n loci] [n scan sets] */

   int   i, j;

   *d = malloc (n_compnts * sizeof (double **));
   alloc_chk (WHERE, *d);
   for (i = 0; i < n_compnts; ++i)
   {
      (*d)[i] = malloc (NLoci * sizeof (double *));
      alloc_chk (WHERE, (*d)[i]);
      for (j = 0; j < NLoci; ++j)
      {
         (*d)[i][j] = malloc (NScSet[i] * sizeof (double));
         alloc_chk (WHERE, (*d)[i][j]);
      }
   }
   return;
}
void ald_nc_nl_n (double ****d, int n)
{
   /* mallocs double array of size [n components][n loci] [n] */

   int   i, j;

   *d = malloc (n_compnts * sizeof (double **));
   alloc_chk (WHERE, *d);
   for (i = 0; i < n_compnts; ++i)
   {
      (*d)[i] = malloc (NLoci * sizeof (double *));
      alloc_chk (WHERE, (*d)[i]);
      for (j = 0; j < NLoci; ++j)
      {
         (*d)[i][j] = malloc (n * sizeof (double));
         alloc_chk (WHERE, (*d)[i][j]);
      }
   }
}
void fre_n (void **a)
{
   if (!*a)
      return;
   free (*a);
   *a = NULL;

   return;
}
void fre_n_n (void ***a, int n)
{
   int   i;

   if (!*a)
      return;
   for (i = 0; i < n; ++i)
      free ((*a)[i]);
   free (*a);
   *a = NULL;

   return;
}
void fre_nc_n (void ***a)
{
   int   i;

   if (!*a)
      return;
   for (i = 0; i < n_compnts; ++i)
      free ((*a)[i]);
   free (*a);
   *a = NULL;

   return;
}
void fre_nl_n (void ***a)
{
   int   i;

   if (!*a)
      return;
   for (i = 0; i < NLoci; ++i)
      free ((*a)[i]);
   free (*a);
   *a = NULL;

   return;
}
void fre_nc_nl_n (void ****a)
{
   int   i, j;

   if (!*a)
      return;
   for (i = 0; i < n_compnts; ++i)
   {
      for (j = 0; j < NLoci; ++j)
         free ((*a)[i][j]);
      free ((*a)[i]);
   }
   free (*a);
   *a = NULL;

   return;
}
