/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*
 *   Setup function for Morgan main programs:
 *     procParStat()
 */

#include "parsie.h"

/**
 * Process the parameter file and set corresponding globals
 * (these start with capital letters, and are found in globs.h)
 */
void procParStat (int argc, char **argv, char *ppn)
{

   /* parse input parameter statements  */
   if (!proc_pars (argc, argv, ppn))
      exit (EXIT_FAILURE);

   prntStatusPP ();

   /* check statements for completeness and consistency.
    * Set up global parameters and data structures     */

   if (IS_MARKERDROP || IS_LM_AUTO || IS_LM_IBDTESTS || IS_LM_MAP || IS_LM_PVAL)
   {
      if (!proc_auto ())
         exit (EXIT_FAILURE);
   }

   if (IS_LM_BAYES || IS_LM_LODS || IS_LM_MARKERS || IS_LM_SCHNELL
       || IS_LM_TWOQTL)
   {
      if (!proc_lods ())
         exit (EXIT_FAILURE);
   }

#if 0
   /* check for 2-D graphical display option */
   if (IS_LM_AUTO)
      if (!proc_disp ())
         exit (EXIT_FAILURE);
#endif

   /* set up for PolyEM programs */
   if (IS_BIVAR || IS_MULTIVAR || IS_UNIBIG || IS_UNIVAR)
   {
      if (!proc_poly ())
         exit (EXIT_FAILURE);
   }

   /* set up pedigree file parameters */
   if (!proc_fils ())
      exit (EXIT_FAILURE);

   /* set up seed parameters */
   if (!proc_seeds ())
      exit (EXIT_FAILURE);

   prntStatusCP ();

   return;
}
