/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/* void  full_seg_prob(), double diff_seg_prob(), 
 * double lik_rat(),      double full_pen_seg_lod(),
 * void  alloc_logprob(), void  free_logprob()
 *
 *  Full log-probability of switches and data for a given pedigree component 
 *    --used to track mixing and provide diagnostics                        
 *  also, same for a given meiosis, and for a given locus interval,
 *    for tracking in MCMC updates 
 *  also total combined penetrance and segregation log-probability 
 *    still used ?                               Updated, EAT Nov 2000 
 *  Apparently NOT still used !!                          EAT, Apr 2010;
 *       log_pen_prob diagnostics now computed directly from prob array;
 *       see routins in mcmc_lods.c files.
 *
 *  addition of diff_seg_prob, and other updates,         EAT Dec 00 
 *  added seg_prob, then combined diff_seg_prob and seg_prob into
 *       new diff_seg_prob,                               EAT Jan 01    
 *  replaced fixed arrays with mallocs                    MAJ Aug 01
 */
#include "lsample.h"
#include "msample.h"

static int **new_count;                 /* full_seg_prob:   [NLOC][6]     */
static int **neu_count;                 /* diff_seg_prob:   [NLOC][4]     */
static int **sim_count;                 /* diff_seg_prob:   [NLOC][4]     */
static int ***tempcount;                /* full_pen_seg_lod [NC][NLOC][6] */
static double **templogprob;            /* full_pen_seg_lod [NC][NLOC]    */

void full_seg_prob (int nl, int c, int ***recount, double **log_prob_seg)
 /*  Compute the full log-prob for given set of switches on each component 
  *  of the pedigree at the sim chromosome    */
{
   int   i, jj, jl, jr, sx;
   double pp = 0.0;
   int   iptr;

   for (jj = 0; jj < nl; ++jj)
      for (i = 0; i < 6; ++i)
         new_count[jj][i] = 0;

   for (iptr = 0; iptr < cmplist[c + 1].nmeioses; ++iptr)
   {
      i = cmplist[c + 1].meioses[iptr]; /* note components indexed 1,... */
      sx = meiosis[i].sex;
      i = (meiosis[i].ind).index;

      for (jj = 0; jj < nl; ++jj)
      {
         jr = chromo[0][jj + 1];
         jl = chromo[0][jj];
         if (sx == 0)
         {
            ++new_count[jj][3];         /* total female count */
            if (xx0[jl][i] == xx0[jr][i])
               ++new_count[jj][5];      /*non-rec */
            else
               ++new_count[jj][4];      /* recom  */
         }
         else
         {
            ++new_count[jj][0];         /* total male count */
            if (xx1[jl][i] == xx1[jr][i])
               ++new_count[jj][2];      /* non-rec */
            else
               ++new_count[jj][1];      /* recom  */
         }
      }
   }

   for (jj = 0; jj < nl; ++jj)
   {
      pp = new_count[jj][5] * log (1.0 - xecomo[0][0][jj]);
      pp += new_count[jj][4] * log (xecomo[0][0][jj]);
      pp += new_count[jj][2] * log (1.0 - xecomo[1][0][jj]);
      pp += new_count[jj][1] * log (xecomo[1][0][jj]);
      log_prob_seg[c][jj] = pp;
      for (i = 0; i < 6; ++i)
         recount[c][jj][i] += new_count[jj][i];
   }
}

double diff_seg_prob (int nl, int c, int h, int *intflag, double **log_prob_seg)
/*  Compute the log-prob difference for given set of 
 *  switches on each component of the pedigree,
 *  between chromosome h and the sim chromosome       
 *  01/11/01, EAT: 
 *  combine diff_seg_prob and seg_prob routines;    
 *  added intflag[]  flags if interval needed for current computation; 
 *  also now return total 
 */
{
   int   i, jj, jl, jr, sx, jsl, jsr;
   int   iptr;
   double ppp = 0.0;
   double pp = 0.0;

   for (jj = 0; jj < nl; ++jj)
      for (i = 0; i < 4; ++i)
      {
         neu_count[jj][i] = 0;
         sim_count[jj][i] = 0;
      }

   for (iptr = 0; iptr < cmplist[c + 1].nmeioses; ++iptr)
   {
      i = cmplist[c + 1].meioses[iptr]; /* note components indexed 1,... */
      sx = meiosis[i].sex;
      i = (meiosis[i].ind).index;

      for (jj = 0; jj < nl; ++jj)
      {
         if ((lpflag[sx][h][jj]) && (intflag[jj]))
         {
            jr = chromo[h][jj + 1];
            jsr = chromo[0][jj + 1];
            jl = chromo[h][jj];
            jsl = chromo[0][jj];
            if (sx == 0)
            {
               if (xx0[jl][i] == xx0[jr][i])
                  ++neu_count[jj][3];   /*non-rec */
               else
                  ++neu_count[jj][2];   /* recom  */
               if (xx0[jsl][i] == xx0[jsr][i])
                  ++sim_count[jj][3];   /*non-rec */
               else
                  ++sim_count[jj][2];   /* recom  */
            }
            else
            {
               if (xx1[jl][i] == xx1[jr][i])
                  ++neu_count[jj][1];   /* non-rec */
               else
                  ++neu_count[jj][0];   /* recom  */
               if (xx1[jsl][i] == xx1[jsr][i])
                  ++sim_count[jj][1];   /* non-rec */
               else
                  ++sim_count[jj][0];   /* recom  */
            }
         }
      }
   }

   for (jj = 0; jj < nl; ++jj)
   {
      pp = 0.0;
      if (intflag[jj])
      {
         if (lpflag[0][h][jj])
         {
            pp += neu_count[jj][3] * log (1.0 - xecomo[0][h][jj]);
            pp += neu_count[jj][2] * log (xecomo[0][h][jj]);
            pp -= sim_count[jj][3] * log (1.0 - xecomo[0][0][jj]);
            pp -= sim_count[jj][2] * log (xecomo[0][0][jj]);
         }

         if (lpflag[1][h][jj])
         {
            pp += neu_count[jj][1] * log (1.0 - xecomo[1][h][jj]);
            pp += neu_count[jj][0] * log (xecomo[1][h][jj]);
            pp -= sim_count[jj][1] * log (1.0 - xecomo[1][0][jj]);
            pp -= sim_count[jj][0] * log (xecomo[1][0][jj]);
         }

         log_prob_seg[c][jj] = pp;
         ppp += pp;
      }
   }
   return (ppp);
}

double lik_rat (int nl, int ip, int sx, int h1)
/* likelihood ratio for a single meiosis individual ip, sex sx,
 *   between two haplotype sets, h1, and the sim-set 0,
 *   counting only those intervals where the haplotypes differ 
 *   -- modified to be LOG-likelihod ratio;  EAT 12/2000   
 *   -- modified to h2=0, and to use lpflag;  EAT 12/2000   
 */
{
   int   jj, jr, jl;
   double pp;
   int **xx;

   if (sx == 0)
      xx = xx0;
   else
      xx = xx1;

   pp = 0.0;
   for (jj = 0; jj < nl; ++jj)
   {
      if (lpflag[sx][h1][jj])
      {
         jr = chromo[h1][jj + 1];
         jl = chromo[h1][jj];
         if (xx[jl][ip] == xx[jr][ip])
            pp += log (1.0 - xecomo[sx][h1][jj]);
         else
            pp += log (xecomo[sx][h1][jj]);
         jr = chromo[0][jj + 1];
         jl = chromo[0][jj];
         if (xx[jl][ip] == xx[jr][ip])
            pp -= log (1.0 - xecomo[sx][0][jj]);
         else
            pp -= log (xecomo[sx][0][jj]);
      }
   }
   return (pp);
}

double full_pen_seg_lod (int nl, int nsw, int ngenes, int err, int ***y)
      /* total logprob of the data log P(Y,S)  */
{
   int   j, c, k;
   double psegp, ptot;

   psegp = 0.0;
   ptot = 0.0;

   for (c = 0; c < n_compnts; ++c)
      for (j = 0; j < nl; ++c)
      {
         for (k = 0; k < 6; ++k)
            tempcount[c][j][k] = 0;
         templogprob[c][j] = 0;
      }

   full_seg_prob (nl, c, tempcount, templogprob);

   for (c = 0; c < n_compnts; ++c)
   {
      for (j = 0; j <= nl; ++j)
      {                                 /* earlier version of this was BUG; fixed EAT 4/2010 */
         psegp = seg_pen2 (c, loci + j, ngenes, &err);
         if (!err)
            ptot += log (psegp);
         else
         {
            printf ("\n seg_pen error in full_seg_prob");
            exit (3);
         }
      }
      for (j = 0; j < nl; ++j)
         ptot += templogprob[c][nl];
   }

   return (ptot);
}

void alloc_logprob (void)
{
   ali_nl_n (&new_count, 6);
   ali_nl_n (&neu_count, 4);
   ali_nl_n (&sim_count, 4);
   ali_nc_nl_n (&tempcount, 6);
   ald_nc_nl (&templogprob);

   return;
}

void free_logprob (void)
{
   if (!new_count)
      return;

   fre_nl_n ((void *) &new_count);
   fre_nl_n ((void *) &neu_count);
   fre_nl_n ((void *) &sim_count);
   fre_nc_nl_n ((void *) &tempcount);
   fre_nc_n ((void *) &templogprob);
   new_count = NULL;

   return;
}

void full_seg_prob_cmp (int nl, int c, int **recount, double *log_prob_seg)
 /*  Compute the full log-prob for given set of switches on each component 
  *  of the pedigree at the sim chromosome    */
{
   int   i, jj, jl, jr, sx;
   double pp = 0.0;
   int   iptr;

   for (jj = 0; jj < nl; ++jj)
      for (i = 0; i < 6; ++i)
         new_count[jj][i] = 0;

   for (iptr = 0; iptr < cmplist[c + 1].nmeioses; ++iptr)
   {
      i = cmplist[c + 1].meioses[iptr]; /* note components indexed 1,... */
      sx = meiosis[i].sex;
      i = (meiosis[i].ind).index;

      for (jj = 0; jj < nl; ++jj)
      {
         jr = chromo[0][jj + 1];
         jl = chromo[0][jj];
         if (sx == 0)
         {
            ++new_count[jj][3];         /* total female count */
            if (xx0[jl][i] == xx0[jr][i])
               ++new_count[jj][5];      /*non-rec */
            else
               ++new_count[jj][4];      /* recom  */
         }
         else
         {
            ++new_count[jj][0];         /* total male count */
            if (xx1[jl][i] == xx1[jr][i])
               ++new_count[jj][2];      /* non-rec */
            else
               ++new_count[jj][1];      /* recom  */
         }
      }
   }

   for (jj = 0; jj < nl; ++jj)
   {
      pp = new_count[jj][5] * log (1.0 - xecomo[0][0][jj]);
      pp += new_count[jj][4] * log (xecomo[0][0][jj]);
      pp += new_count[jj][2] * log (1.0 - xecomo[1][0][jj]);
      pp += new_count[jj][1] * log (xecomo[1][0][jj]);
      log_prob_seg[jj] = pp;
      for (i = 0; i < 6; ++i)
         recount[jj][i] += new_count[jj][i];
   }
}
