/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "lsample.h"
#include "mulmsample.h"

/* cmp = 0: Consider all components simultaneously.                           */
/* cmp > 0: Indicates which component to consider.                            */
/* mul = 0: M-sampler; otherwise, MM-sampler.                                 */
/* ntp =    Number of test positions.                                         */

void mcmc_lods (int cmp, int mul, int ntp, CTP_PTR p_ctp, double **prob,
                double *mcmc_lod, double *mcmc_sd)
{
   static FILE *oScorF = NULL;

   RLD_PTR p_rld;
   MSD_PTR p_msd;

   int   nctemp;                        /* Backup copy of original n_compnts. */
   int   scored_iterations;             /* Number of scored iterations.       */

   /* Open output scores file, if requested to do so.                         */
   if (FilIt && !oScorF)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   nctemp = n_compnts;
   if (cmp)
      n_compnts = 1;

   /* Call set_batch_size to make sure that the scoring interval evenly       */
   /* divides the realizations count and that the BatchSize global variable   */
   /* is set to a positive number that evenly divides the number of scored    */
   /* iterations.                                                             */

   set_batch_size ();
   printf ("\n");
   fflush (stdout);

   /* Moved down initial peeling after alloc_rld etc: EAT July 2004.          */

   p_rld = alloc_rld ("CPBS", ntp, NBtch);

   set_simple_trans (loci, -1, -1, 0.5, 0.5, 0.5, 0.5);

   if (TraitTyp != QUANTITATIVE)
      p_rld->base_lik = locus_mjrpeel_cmp (cmp, 1.0, 1.0, 0, loci);
   else
      p_rld->base_lik = locus_mjrpeel_cmp (cmp, loci[0].ve, 1.0, 0, loci);

   /* Checks for zero probability in peeling operation.                       */
   if (p_rld->base_lik == -DBL_MAX)
      give_up_where (WHERE, ERS_zpp);

   printf (" Base log-likelihood log_10 P(Y_T) = %e\n",
           p_rld->base_lik / log (10.0));
   printf ("\n\n");
   fflush (stdout);

   p_msd = alloc_msd ();
   alloc_logprob ();

   /* Now call do_mcmc_sampling to do the MCMC sampling and data collection.  */

   do_mcmc_sampling (cmp, mul, ntp, &scored_iterations, prob, mcmc_lod, mcmc_sd,
                     p_ctp, p_rld, p_msd, oScorF);

   /* Call the mcmc_lods_results subroutine to compute, print and return the  */
   /* mcmc_lods subroutine final results.                                     */

   mcmc_lods_results (p_rld, p_msd, p_ctp, ntp, cmp, scored_iterations,
                      mcmc_lod, mcmc_sd);

   free_logprob ();
   free_msd (p_msd);
   free_rld (p_rld);

   n_compnts = nctemp;

   if (FilIt && (!cmp || (cmp == n_compnts)))
      fclose (oScorF);

   return;
}
